@@ -10,7 +10,7 @@ use syn::*;
 use ApiType::*;
 
 use crate::api_types::*;
-use crate::generator_rust::EXECUTOR_NAME;
+use crate::generator_rust::HANDLER_NAME;
 
 type StructMap<'a> = HashMap<String, &'a ItemStruct>;
 
@@ -34,7 +34,7 @@ impl<'a> Parser<'a> {
     fn parse(mut self, source_rust_content: &str, src_fns: Vec<&ItemFn>) -> ApiFile {
         let funcs = src_fns.iter().map(|f| self.parse_function(f)).collect();
 
-        let has_executor = source_rust_content.contains(EXECUTOR_NAME);
+        let has_executor = source_rust_content.contains(HANDLER_NAME);
 
         ApiFile {
             funcs,
@@ -54,6 +54,9 @@ impl<'a> Parser<'a> {
         let func_name = ident_to_string(&sig.ident);
 
         let mut inputs = Vec::new();
+        let mut output = None;
+        let mut mode = ApiFuncMode::Normal;
+
         for sig_input in &sig.inputs {
             if let FnArg::Typed(ref pat_type) = sig_input {
                 let name = if let Pat::Ident(ref pat_ident) = *pat_type.pat {
@@ -61,32 +64,40 @@ impl<'a> Parser<'a> {
                 } else {
                     panic!("unexpected pat_type={:?}", pat_type)
                 };
+                let type_string = type_to_string(&pat_type.ty);
 
-                let ty = self.parse_type(&type_to_string(&pat_type.ty));
-                inputs.push(ApiField {
-                    name: ApiIdent::new(name),
-                    ty,
-                });
+                if let Some(stream_sink_inner_type) = self.try_parse_stream_sink(&type_string) {
+                    output = Some(stream_sink_inner_type);
+                    mode = ApiFuncMode::Stream;
+                } else {
+                    inputs.push(ApiField {
+                        name: ApiIdent::new(name),
+                        ty: self.parse_type(&type_string),
+                    });
+                }
             } else {
                 panic!("unexpected sig_input={:?}", sig_input);
             }
         }
 
-        let output = if let ReturnType::Type(_, ty) = &sig.output {
-            let type_string = type_to_string(ty);
-            if let Some(inner) = CAPTURE_RESULT.captures(&type_string) {
-                self.parse_type(&inner)
+        if output.is_none() {
+            output = Some(if let ReturnType::Type(_, ty) = &sig.output {
+                let type_string = type_to_string(ty);
+                if let Some(inner) = CAPTURE_RESULT.captures(&type_string) {
+                    self.parse_type(&inner)
+                } else {
+                    panic!("unsupported type_string: {}", type_string);
+                }
             } else {
-                panic!("unsupported type_string: {}", type_string);
-            }
-        } else {
-            panic!("unsupported output: {:?}", sig.output);
-        };
+                panic!("unsupported output: {:?}", sig.output);
+            });
+        }
 
         ApiFunc {
             name: func_name,
             inputs,
-            output,
+            output: output.expect("unsupported output"),
+            mode,
         }
     }
 
@@ -96,10 +107,21 @@ impl<'a> Parser<'a> {
             .or_else(|| ApiTypeDelegate::try_from_rust_str(ty).map(Delegate))
             .or_else(|| self.try_parse_list(ty))
             .or_else(|| self.try_parse_box(ty))
+            .or_else(|| self.try_parse_option(ty))
             .or_else(|| self.try_parse_struct(ty))
             .unwrap_or_else(|| panic!("parse_type failed for ty={}", ty))
     }
 
+    fn try_parse_stream_sink(&mut self, ty: &str) -> Option<ApiType> {
+        lazy_static! {
+            static ref CAPTURE_STREAM_SINK: GenericCapture = GenericCapture::new("StreamSink");
+        }
+
+        CAPTURE_STREAM_SINK
+            .captures(ty)
+            .map(|inner| self.parse_type(&inner))
+    }
+
     fn try_parse_list(&mut self, ty: &str) -> Option<ApiType> {
         lazy_static! {
             static ref CAPTURE_VEC: GenericCapture = GenericCapture::new("Vec");
@@ -128,6 +150,32 @@ impl<'a> Parser<'a> {
         })
     }
 
+    fn try_parse_option(&mut self, ty: &str) -> Option<ApiType> {
+        lazy_static! {
+            static ref CAPTURE_OPTION: GenericCapture = GenericCapture::new("Option");
+        }
+
+        CAPTURE_OPTION.captures(ty).map(|inner| {
+            let inner_option = CAPTURE_OPTION.captures(&inner);
+            if let Some(inner_option) = inner_option {
+                panic!(
+                    "Nested optionals without indirection are not supported. (Option<Option<{}>>)",
+                    inner_option
+                );
+            };
+            match self.parse_type(&inner) {
+                Primitive(prim) => ApiType::Optional(ApiTypeOptional::new_prim(prim)),
+                st @ StructRef(_) => {
+                    ApiType::Optional(ApiTypeOptional::new_ptr(Boxed(Box::new(ApiTypeBoxed {
+                        inner: st,
+                        exist_in_real_api: false,
+                    }))))
+                }
+                other => ApiType::Optional(ApiTypeOptional::new_ptr(other)),
+            }
+        })
+    }
+
     fn try_parse_struct(&mut self, ty: &str) -> Option<ApiType> {
         if !self.src_struct_map.contains_key(ty) {
             return None;
@@ -214,7 +262,7 @@ struct GenericCapture {
 
 impl GenericCapture {
     pub fn new(cls_name: &str) -> Self {
-        let regex = Regex::new(&*format!(".*{}<([a-zA-Z0-9_<>]+)>$", cls_name)).unwrap();
+        let regex = Regex::new(&*format!("^[^<]*{}<([a-zA-Z0-9_<>]+)>$", cls_name)).unwrap();
         Self { regex }
     }
 
